use std::collections::VecDeque;

fn get_lines(raw: &str) -> Vec<Vec<char>> {
    raw.split('\n').map(|line| line.chars().collect()).collect()
}

fn filter_lines(raw: &Vec<Vec<char>>) -> Vec<VecDeque<char>> {
    raw.iter()
        .map(|line| {
            let line: Vec<char> = line
                .into_iter()
                .filter(|c| c.is_digit(10))
                .map(|c| c.to_owned())
                .collect();
            VecDeque::from(line)
        })
        .collect()
}

fn get_values(raw: &mut Vec<VecDeque<char>>) -> Vec<usize> {
    raw.iter_mut()
        .map(|line| {
            let first = match line.pop_front() {
                Some(c) => c,
                None => '0',
            };
            let last = match line.pop_back() {
                Some(c) => c,
                None => first,
            };
            let str: String = Vec::from([first, last]).into_iter().collect();

            return usize::from_str_radix(&str, 10).unwrap();
        })
        .collect()
}

fn sum(raw: Vec<usize>) -> usize {
    raw.into_iter().reduce(|accum, item| accum + item).unwrap()
}

fn main() {
    let file_str = include_str!("input.txt");
    let lines = get_lines(file_str);
    let mut lines = filter_lines(&lines);
    let values = get_values(&mut lines);

    println!("Total: {}", sum(values));
}
